@@ -1,7 +1,7 @@
 /* -*- indent-tabs-mode: t; tab-width: 8; c-basic-offset: 8; -*- */
 
 /*
-   Copyright (c) 2010 - 2013 Thomas Schmitt <scdbackup@gmx.net>
+   Copyright (c) 2010 - 2016 Thomas Schmitt <scdbackup@gmx.net>
    Provided under GPL version 2 or later.
 */
 
@@ -45,7 +45,7 @@ sg_initialize()         performs global initialization of the SCSI transport
                         software components.
 
 sg_shutdown()           performs global finalizations and releases golbally
-                        aquired resources.
+                        acquired resources.
 
 sg_give_next_adr()      iterates over the set of potentially useful drive 
                         address strings.
@@ -54,7 +54,7 @@ scsi_enumerate_drives() brings all available, not-whitelist-banned, and
                         accessible drives into libburn's list of drives.
 
 sg_dispose_drive()      finalizes adapter specifics of struct burn_drive
-                        on destruction. Releases resources which were aquired
+                        on destruction. Releases resources which were acquired
                         underneath scsi_enumerate_drives().
  
 sg_drive_is_open()      tells wether libburn has the given drive in use.
@@ -78,7 +78,7 @@ burn_os_is_2k_seekrw()  tells whether the given path leads to a file object
 
 burn_os_stdio_capacity()  estimates the emulated media space of stdio-drives.
 
-burn_os_open_track_src()  opens a disk file in a way that allows best
+burn_os_open_track_src()  opens a disk file in a way that offers best
                         throughput with file reading and/or SCSI write command
                         transmission.
 
@@ -116,6 +116,7 @@ Send feedback to libburn-hackers@pykix.org .
 #include <sys/statvfs.h>
 #endif /* Libburn_os_has_stavtfS */
 
+#include <volmgt.h>
 #include <sys/dkio.h>
 #include <sys/vtoc.h>
 
@@ -200,13 +201,13 @@ static int decode_btl_number(char **cpt, int stopper, int *no)
 }
 
 
-/* Read bus, target, lun from name "cXtYdZs2".
+/* Read bus, target, lun from name "cXtYdZs2" or "cXtYdZ/...".
    Return 0 if name is not of the desired form.
 */
 static int decode_btl_solaris(char *name, int *busno, int *tgtno, int *lunno,
                         int flag)
 {
-  char *cpt;
+  char *cpt, *cpt_mem;
   int ret;
 
   *busno = *tgtno = *lunno = -1;
@@ -219,9 +220,15 @@ static int decode_btl_solaris(char *name, int *busno, int *tgtno, int *lunno,
   ret = decode_btl_number(&cpt, 'd', tgtno);
   if (ret <= 0)
     return ret;
+  cpt_mem = cpt;
   ret = decode_btl_number(&cpt, 's', lunno);
-  if (ret <= 0)
-    return ret;
+  if (ret <= 0) {
+    cpt = cpt_mem;
+    ret = decode_btl_number(&cpt, '/', lunno);
+    if (ret <= 0)
+      return ret;
+    return(1);
+  }
   cpt++;
   if (*cpt != '2' || *(cpt + 1) != 0)
     return 0;
@@ -247,16 +254,64 @@ static int start_enum_cXtYdZs2(burn_drive_enumerator_t *idx, int flag)
 }
 
 
+static int sg_solaris_convert_devname(char *path, char **dev_to_open, int flag)
+{
+	char *sym_name = NULL, *media_name = NULL, *curr_name, *msg = NULL;
+	int ret;
+
+	BURN_ALLOC_MEM(msg, char, 4096);
+
+	BURN_FREE_MEM(*dev_to_open);
+	*dev_to_open = NULL;
+	curr_name = path;
+
+	if (! volmgt_running())
+		goto set_name;
+	sym_name = volmgt_symname(path); 
+	sprintf(msg, "Volume Management symbolic name: '%s' -> %s",
+			path, sym_name == NULL ? "NULL" : sym_name);
+	libdax_msgs_submit(libdax_messenger, -1,
+			0x00000002,
+			LIBDAX_MSGS_SEV_DEBUG, LIBDAX_MSGS_PRIO_HIGH,
+			msg, 0, 0);
+	if (sym_name != NULL)
+		media_name = media_findname(sym_name);
+	else
+		media_name = media_findname(path);
+	if (media_name != NULL)
+		curr_name = media_name;
+	sprintf(msg, "Media name: %s -> %s",
+			sym_name == NULL ? path : sym_name,
+			media_name == NULL ? "NULL" : media_name);
+	libdax_msgs_submit(libdax_messenger, -1,
+			0x00000002,
+			LIBDAX_MSGS_SEV_DEBUG, LIBDAX_MSGS_PRIO_HIGH,
+			msg, 0, 0);
+set_name:
+	BURN_ALLOC_MEM(*dev_to_open, char, strlen(curr_name) + 1);
+	strcpy(*dev_to_open, curr_name);
+	ret = 1;
+ex:
+	if (media_name != NULL)
+		free(media_name);
+	if (sym_name != NULL)
+		free(sym_name);
+	BURN_FREE_MEM(msg);
+	return(ret);
+}
+
+
 static int next_enum_cXtYdZs2(burn_drive_enumerator_t *idx,
 				char adr[], int adr_size, int flag)
 { 
-	int busno, tgtno, lunno, ret, fd = -1, volpath_size = 160;
-	char *volpath = NULL;
+	int busno, tgtno, lunno, ret, fd = -1, volpath_size = 160, os_errno;
+	char *volpath = NULL, *msg = NULL, *dev_to_open = NULL;
 	struct dirent *entry;
 	struct dk_cinfo cinfo;
 	DIR *dir;
 
 	BURN_ALLOC_MEM(volpath, char, volpath_size);
+	BURN_ALLOC_MEM(msg, char, 4096);
 
 	dir = idx->dir;
 	while (1) {
@@ -282,25 +337,68 @@ static int next_enum_cXtYdZs2(burn_drive_enumerator_t *idx,
 
 		sprintf(volpath, "/dev/rdsk/%s", entry->d_name);
 		if (burn_drive_is_banned(volpath))
-	continue; 
-
-		fd = open(volpath, O_RDONLY | O_NDELAY);
-		if (fd < 0)
 	continue;
+		ret = sg_solaris_convert_devname(volpath, &dev_to_open, 0);
+		if (ret <= 0)
+	continue;
+		fd = open(dev_to_open, O_RDONLY | O_NDELAY);
+		if (fd < 0) {
+			os_errno = errno;
+			sprintf(msg, "Could not open '%s' , errno = %d",
+				dev_to_open, os_errno);
+			libdax_msgs_submit(libdax_messenger, -1,
+				0x00000002,
+				LIBDAX_MSGS_SEV_DEBUG, LIBDAX_MSGS_PRIO_HIGH,
+				msg, os_errno, 0);
+	continue;
+		}
 		/* See man dkio */
 		ret = ioctl(fd, DKIOCINFO, &cinfo);
 		close(fd);
-		if (ret < 0)
+		if (ret < 0) {
+			os_errno = errno;
+			sprintf(msg,
+			   "ioctl(DKIOCINFO) failed on drive '%s', errno = %d",
+				volpath, os_errno);
+			libdax_msgs_submit(libdax_messenger, -1,
+				0x00000002,
+				LIBDAX_MSGS_SEV_DEBUG, LIBDAX_MSGS_PRIO_HIGH,
+				msg, os_errno, 0);
 	continue;
-		if (cinfo.dki_ctype != DKC_CDROM)
+		}
+		if (cinfo.dki_ctype != DKC_CDROM) {
+			sprintf(msg,
+				"ioctl(DKIOCINFO) classifies drive '%s' as dki_ctype %ld, not as DKC_CDROM = %ld",
+				volpath, (long int) cinfo.dki_ctype,
+				(long int) DKC_CDROM);
+			libdax_msgs_submit(libdax_messenger, -1,
+				0x00000002,
+				LIBDAX_MSGS_SEV_DEBUG, LIBDAX_MSGS_PRIO_HIGH,
+				msg, 0, 0);
 	continue;
-		if (adr_size <= (int) strlen(volpath))
+		}
+		if (adr_size <= (int) strlen(volpath)) {
+			sprintf(msg,
+				"Device path '%s' too long. (Max. %d)",
+				volpath, adr_size - 1);
+			libdax_msgs_submit(libdax_messenger, -1,
+				0x00000002,
+				LIBDAX_MSGS_SEV_DEBUG, LIBDAX_MSGS_PRIO_HIGH,
+				msg, 0, 0);
 			{ret = -1; goto ex;}
+		}
 		strcpy(adr, volpath);
+		sprintf(msg, "Accepted as valid drive '%s'", volpath);
+		libdax_msgs_submit(libdax_messenger, -1,
+				0x00000002,
+				LIBDAX_MSGS_SEV_DEBUG, LIBDAX_MSGS_PRIO_HIGH,
+				msg, 0, 0);
 		{ret = 1; goto ex;}
 	}
 	ret = 0;
 ex:;
+	BURN_FREE_MEM(dev_to_open);
+	BURN_FREE_MEM(msg);
 	BURN_FREE_MEM(volpath);
 	return ret;
 }
@@ -399,7 +497,7 @@ int sg_initialize(char msg[1024], int flag)
 
 
 /** Performs global finalization of the SCSI transport adapter and eventually
-    needed operating system facilities. Releases globally aquired resources.
+    needed operating system facilities. Releases globally acquired resources.
     @param flag  unused yet, submit 0
     @return      1 = success, <=0 = failure
 */ 
@@ -492,7 +590,7 @@ ex:;
 
 /** Tells whether libburn has the given drive in use or exclusively reserved.
     If it is "open" then libburn will eventually call sg_release() on it when
-    it is time to give up usage resp. reservation.
+    it is time to give up usage and reservation.
 */
 /** Published as burn_drive.drive_is_open() */
 int sg_drive_is_open(struct burn_drive * d)
@@ -509,7 +607,7 @@ int sg_drive_is_open(struct burn_drive * d)
 */  
 int sg_grab(struct burn_drive *d)
 {
-	char *msg = NULL;
+	char *msg = NULL, *dev_to_open = NULL;
 	int os_errno, ret;
 	struct dk_cinfo cinfo;
 
@@ -519,10 +617,16 @@ int sg_grab(struct burn_drive *d)
 		d->released = 0;
 		{ret = 1; goto ex;}
 	}
-	d->fd = open(d->devname, O_RDONLY | O_NDELAY);
+	ret = sg_solaris_convert_devname(d->devname, &dev_to_open, 0);
+	if (ret <= 0)
+		goto ex;
+	d->fd = open(dev_to_open, O_RDONLY | O_NDELAY);
 	if (d->fd == -1) {
 		os_errno = errno;
-		sprintf(msg, "Could not grab drive '%s'", d->devname);
+		sprintf(msg, "Could not grab drive '%s'",
+			d->devname);
+		if (strcmp(d->devname, dev_to_open))
+			sprintf(msg + strlen(msg), " via '%s'", dev_to_open);
 		libdax_msgs_submit(libdax_messenger, d->global_index,
 			0x00020003,
 			LIBDAX_MSGS_SEV_SORRY, LIBDAX_MSGS_PRIO_HIGH,
@@ -530,10 +634,27 @@ int sg_grab(struct burn_drive *d)
 		{ret = 0; goto ex;}
 	}
 	ret = ioctl(d->fd, DKIOCINFO, &cinfo);
-	if (ret < 0)
+	if (ret < 0) {
+		os_errno = errno;
+		sprintf(msg, "ioctl(DKIOCINFO) failed on drive '%s'",
+			d->devname);
+		libdax_msgs_submit(libdax_messenger, d->global_index,
+			0x00000002,
+			LIBDAX_MSGS_SEV_DEBUG, LIBDAX_MSGS_PRIO_HIGH,
+			msg, os_errno, 0);
 		goto revoke;
-	if (cinfo.dki_ctype != DKC_CDROM)
+	}
+	if (cinfo.dki_ctype != DKC_CDROM) {
+		sprintf(msg,
+			"ioctl(DKIOCINFO) classifies drive '%s' as dki_ctype %ld, not as DKC_CDROM = %ld",
+			d->devname, (long int) cinfo.dki_ctype,
+			(long int) DKC_CDROM);
+		libdax_msgs_submit(libdax_messenger, d->global_index,
+			0x00000002,
+			LIBDAX_MSGS_SEV_DEBUG, LIBDAX_MSGS_PRIO_HIGH,
+			msg, 0, 0);
 		goto revoke;
+	}
 
 	/* >>> obtain eventual locks */;
 
@@ -548,6 +669,7 @@ revoke:;
 			msg, 0, 0);
 	ret = 0;
 ex:;
+	BURN_FREE_MEM(dev_to_open);
 	BURN_FREE_MEM(msg);
 	return ret;
 }
@@ -632,6 +754,9 @@ int sg_issue_command(struct burn_drive *d, struct command *c)
 	cgc.uscsi_rqlen = sizeof(c->sense);
 	cgc.uscsi_rqbuf = (caddr_t) c->sense;
 
+	/* ts B90523 : Record effective transfer length request for debugging*/
+	c->dxfer_len = cgc.uscsi_buflen;
+
 	/* retry-loop */
 	start_time = time(NULL);
 	for(i = 0; !done; i++) {
@@ -677,6 +802,8 @@ int sg_issue_command(struct burn_drive *d, struct command *c)
 						start_time, timeout_ms, i, 0);
 		if (d->cancel)
 			done = 1;
+		if (!done)
+			spc_register_retry(c);
 					
 	} /* end of retry-loop */
 
@@ -759,7 +886,7 @@ int sg_is_enumerable_adr(char* adr)
 
 
 /* Return 1 if the given path leads to a regular file or a device that can be
-   seeked, read, and possibly written with 2 kB granularity. 
+   fseeked, read, and possibly written with 2 kB granularity. 
 */
 int burn_os_is_2k_seekrw(char *path, int flag)
 {
