@@ -2,8 +2,9 @@
 /*
   Class struct of libisoburn.
 
-  Copyright 2007 - 2017 Vreixo Formoso Lopes <metalpain2002@yahoo.es>
-                    and Thomas Schmitt <scdbackup@gmx.net>
+  Copyright 2007 - 2009 Vreixo Formoso Lopes <metalpain2002@yahoo.es>
+  Copyright 2007 - 2019 Thomas Schmitt <scdbackup@gmx.net>
+
   Provided under GPL version 2 or later.
 */
 
@@ -691,12 +692,19 @@ struct isoburn_imgen_opts {
     char *efi_boot_partition;
     int efi_boot_part_flag;
 
-    /* Eventual disk file paths of prepared images which shall be appended
+    /* Disk file paths of prepared images which shall be appended
        after the ISO image and described by partiton table entries in a MBR.
+       NULL means unused.
     */
     char *appended_partitions[Libisoburn_max_appended_partitionS];
     uint8_t appended_part_types[Libisoburn_max_appended_partitionS];
     int appended_part_flags[Libisoburn_max_appended_partitionS];
+    uint8_t appended_part_type_guids[Libisoburn_max_appended_partitionS][16];
+
+    /* Flags in case that appended partitions show up in GPT:
+       bit0= appended_part_type_guids[same_index] is valid
+    */
+    uint8_t appended_part_gpt_flags[Libisoburn_max_appended_partitionS];
 
     /* If 1: With appended partitions: create protective MBR and mark by GPT
     */
@@ -714,6 +722,13 @@ struct isoburn_imgen_opts {
     */
     int iso_mbr_part_type;
 
+    /* isoburn_igopt_set_iso_type_guid()
+    */
+    uint8_t iso_gpt_type_guid[16];
+    /* bit0= iso_gpt_type_guid is valid
+    */
+    int iso_gpt_flag;
+
     /* See libisoburn.h isoburn_igopt_set_gpt_guid()
     */
     uint8_t gpt_guid[16];
