@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2014 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2015 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -210,6 +210,7 @@ char *Spotlist__quality_name(int quality, char name[80], int bad_limit,
 {
  if(quality == Xorriso_read_quality_untesteD ||
    quality == Xorriso_read_quality_tao_enD ||
+   (quality == Xorriso_read_quality_md5_mismatcH && quality > bad_limit) ||
    quality == Xorriso_read_quality_off_tracK)
    strcpy(name, "0 ");
  else if(quality <= bad_limit)
@@ -228,14 +229,14 @@ char *Spotlist__quality_name(int quality, char name[80], int bad_limit,
    strcat(name, "valid");
  else if(quality == Xorriso_read_quality_untesteD)
    strcat(name, "untested");
+ else if(quality == Xorriso_read_quality_md5_mismatcH)
+   strcat(name, "md5_mismatch");
  else if(quality == Xorriso_read_quality_invaliD)
    strcat(name, "invalid");
  else if(quality == Xorriso_read_quality_tao_enD)
    strcat(name, "tao_end");
  else if(quality == Xorriso_read_quality_off_tracK)
    strcat(name, "off_track");
- else if(quality == Xorriso_read_quality_md5_mismatcH)
-   strcat(name, "md5_mismatch");
  else if(quality == Xorriso_read_quality_unreadablE)
    strcat(name, "unreadable");
  else
@@ -385,7 +386,7 @@ int Sectorbitmap_to_file(struct SectorbitmaP *o, char *path, char *info,
      strcpy(msg, "Cannot open path ");
      Text_shellsafe(path, msg+strlen(msg), 0);
    }
-   return(0);
+   ret= 0; goto ex;
  }
 
  l= 0;
@@ -393,7 +394,7 @@ int Sectorbitmap_to_file(struct SectorbitmaP *o, char *path, char *info,
    l= strlen(info);
  if(l > 999999) {
    strcpy(msg, "Info string is longer than 999999 bytes");
-   return(0);
+   ret= 0; goto ex;
  }
  sprintf((char *) buf, "xorriso sector bitmap v2 %-6d\n", l);
 
@@ -678,14 +679,14 @@ int Xorriso_check_media_setup_job(struct XorrisO *xorriso,
        xorriso->check_media_bad_limit= Xorriso_read_quality_valiD;
      else if(strcmp(argv[i] + 10, "untested") == 0)
        xorriso->check_media_bad_limit= Xorriso_read_quality_untesteD;
+     else if(strcmp(argv[i] + 10, "md5_mismatch") == 0)
+       xorriso->check_media_bad_limit= Xorriso_read_quality_md5_mismatcH;
      else if(strcmp(argv[i] + 10, "invalid") == 0)
        xorriso->check_media_bad_limit= Xorriso_read_quality_invaliD;
      else if(strcmp(argv[i] + 10, "tao_end") == 0)
        xorriso->check_media_bad_limit= Xorriso_read_quality_tao_enD;
      else if(strcmp(argv[i] + 10, "off_track") == 0)
        xorriso->check_media_bad_limit= Xorriso_read_quality_off_tracK;
-     else if(strcmp(argv[i] + 10, "md5_mismatch") == 0)
-       xorriso->check_media_bad_limit= Xorriso_read_quality_md5_mismatcH;
      else if(strcmp(argv[i] + 10, "unreadable") == 0)
        xorriso->check_media_bad_limit= Xorriso_read_quality_unreadablE;
      else
@@ -922,11 +923,13 @@ ex:;
  strcat(report, " ");
  strcat(report, xorriso->list_delimiter);
  Checkmediajob_destroy(&dflt, 0);
- sprintf(default_report, "-check_media_defaults reset=now %s",
-         xorriso->list_delimiter);
+ if(default_report != NULL) {
+   sprintf(default_report, "-check_media_defaults reset=now %s",
+           xorriso->list_delimiter);
+   if(ret > 0 && strcmp(report, default_report) == 0)
+     ret= 2;
+ }
  Xorriso_free_meM(default_report);
- if(ret > 0 && strcmp(report, default_report) == 0)
-   return(2);
  return(ret);
 }
 
